@@ -59,7 +59,6 @@ def AgGrid(
     enable_enterprise_modules: bool
     | Literal["enterpriseOnly", "enterprise+AgCharts"] = False,
     license_key: str = None,
-    try_to_convert_back_to_original_types: bool = False,
     conversion_errors: str = "coerce",
     columns_state=None,
     theme: str
@@ -75,6 +74,7 @@ def AgGrid(
     custom_jscode_for_grid_return: JsCode = None,
     should_grid_return: JsCode = None,
     use_json_serialization: bool | Literal["auto"] = "auto",
+    server_sync_strategy: Literal["client_wins", "server_wins"] = "client_wins",
     **default_column_parameters,
 ) -> AgGridReturn:
     """Renders a DataFrame using AgGrid.
@@ -104,10 +104,6 @@ def AgGrid(
         See: https://www.ag-grid.com/react-data-grid/grid-size/#dom-layout
         Defaults to 400.
 
-    fit_columns_on_grid_load : bool, optional
-        DEPRECATED. Use gridOptions autoSizeStrategy instead.
-        See: https://www.ag-grid.com/javascript-data-grid/column-sizing/#auto-sizing-columns
-
     update_mode : GridUpdateMode, optional
         DEPRECATED. Use update_on parameter instead.
         Defines how the grid sends results back to Streamlit.
@@ -136,10 +132,6 @@ def AgGrid(
         License key for AG Grid Enterprise features.
         Defaults to None.
 
-    try_to_convert_back_to_original_types : bool, optional
-        Attempts to convert grid data back to original DataFrame types.
-        Defaults to False.
-
     conversion_errors : str, optional
         How to handle type conversion errors:
             - 'raise': Raises exception on conversion failure
@@ -244,6 +236,18 @@ def AgGrid(
         or False for strict type checking.
         Defaults to 'auto'.
 
+    server_sync_strategy : Literal['client_wins', 'server_wins'], optional
+        Controls data synchronization behavior between server and client:
+
+        - 'client_wins' (default): After first edit, grid ignores server data updates
+          and maintains local edits. Standard behavior for interactive editing.
+        - 'server_wins': Server data always overwrites the grid, including edited cells.
+          Useful when server data should be the single source of truth.
+
+        When using 'server_wins', consider intercepting grid results with session_state
+        to preserve user edits before re-rendering.
+        Defaults to 'client_wins'.
+
     **default_column_parameters
         Additional parameters passed to gridOptions.defaultColDef.
 
@@ -270,11 +274,31 @@ def AgGrid(
 
     ###Deprecation Warnings
     # Check for deprecated reload_data parameter
-    if 'reload_data' in default_column_parameters:
-        default_column_parameters.pop('reload_data')
+    if "reload_data" in default_column_parameters:
+        default_column_parameters.pop("reload_data")
         warning_key = "reload_data_deprecated"
         if warning_key not in _shown_deprecation_warnings:
-            warnings.warn("The 'reload_data' parameter has been removed and has no effect.", DeprecationWarning, stacklevel=2)
+            warnings.warn(
+                "The 'reload_data' parameter has been removed and has no effect.",
+                DeprecationWarning,
+                stacklevel=2,
+            )
+            _shown_deprecation_warnings.add(warning_key)
+
+    try_to_convert_back_to_original_types: bool = True
+    # Deprecated parameter handling for backward compatibility
+    if "try_to_convert_back_to_original_types" in default_column_parameters:
+        try_to_convert_back_to_original_types = default_column_parameters.pop(
+            "try_to_convert_back_to_original_types"
+        )
+        warning_key = "try_to_convert_back_to_original_types_deprecated"
+        if warning_key not in _shown_deprecation_warnings:
+            warnings.warn(
+                "The 'try_to_convert_back_to_original_types' parameter is deprecated and will be removed in a future version. "
+                "The component now handles type preservation automatically where appropriate.",
+                DeprecationWarning,
+                stacklevel=2,
+            )
             _shown_deprecation_warnings.add(warning_key)
 
     ##Parses Themes
@@ -351,9 +375,14 @@ def AgGrid(
     if custom_jscode_for_grid_return is not None:
         custom_jscode_for_grid_return = custom_jscode_for_grid_return.js_code
         allow_unsafe_jscode = True
+    
+    # Process JsCode for should_grid_return
+    if should_grid_return is not None:
+        should_grid_return = should_grid_return.js_code
+        allow_unsafe_jscode = True
 
     # parse data and gridOptions
-    data, gridOptions = _parse_data_and_grid_options(
+    data, gridOptions, frame_dtypes = _parse_data_and_grid_options(
         data,
         gridOptions,
         default_column_parameters,
@@ -361,10 +390,6 @@ def AgGrid(
         use_json_serialization,
     )
 
-    frame_dtypes = (
-        [i.kind for i in data.dtypes.values.tolist()] if data is not None else []
-    )
-
     if not isinstance(data, pd.DataFrame):
         try_to_convert_back_to_original_types = False
 
@@ -373,8 +398,11 @@ def AgGrid(
     if height is None:
         gridOptions["domLayout"] = "autoHeight"
 
-    if  default_column_parameters.pop("fit_columns_on_grid_load", False):
-        warnings.warn("fit_columns_on_grid_load is deprecated. Use gridOptions autoSizeStrategy instead.", DeprecationWarning)
+    if default_column_parameters.pop("fit_columns_on_grid_load", False):
+        warnings.warn(
+            "fit_columns_on_grid_load is deprecated. Use gridOptions autoSizeStrategy instead.",
+            DeprecationWarning,
+        )
         gridOptions["autoSizeStrategy"] = {"type": "fitGridWidth"}
 
     # Create collector based solely on data_return_mode
@@ -392,8 +420,9 @@ def AgGrid(
 
         collector = LegacyCollector(
             data_return_mode=data_return_mode,
-            try_to_convert_back_to_original_types=try_to_convert_back_to_original_types,
+            try_to_convert_back_to_original_types=True,
             conversion_errors=conversion_errors,
+            frame_dtypes=frame_dtypes,
         )
 
     # Create initial response object that callbacks can safely reference
@@ -408,6 +437,8 @@ def AgGrid(
     response = collector.create_initial_response(
         original_data=original_data,
         grid_options=gridOptions,
+        try_to_convert_back_to_original_types=try_to_convert_back_to_original_types,
+        conversion_errors=conversion_errors,
     )
 
     if callback and not key:
@@ -416,16 +447,20 @@ def AgGrid(
     elif key and not callback:
         # This allows the table to keep its state up to date (eg #176)
         def _inner_callback():
-            component_value = st.session_state[key]
-            # Update the existing response object with new component value
-            collector.update_response(response, component_value)
+            component_value = st.session_state.get(key)
+            # Update the existing response object with new component value and store the wrapped response
+            
+            #updating the session key causes the callback to trigger always. Hot fix in 1.2.0.post2
+            #st.session_state[key] = updated_response
 
     elif callback and key:
         # User defined callback
         def _inner_callback():
-            component_value = st.session_state[key]
-            # Update the existing response object with new component value
+            component_value = st.session_state.get(key)
+            # Update the existing response object with new component value and store the wrapped response
             updated_response = collector.update_response(response, component_value)
+            #updating the session key causes the callback to trigger always. Hot fix in 1.2.0.post2
+            #st.session_state[key] = updated_response
             return callback(updated_response)
     else:
         _inner_callback = None
@@ -472,7 +507,7 @@ def AgGrid(
         gridOptions=gridOptions,
         height=height,
         data_return_mode=data_return_mode,
-        frame_dtypes=frame_dtypes,
+        frame_dtypes=str(frame_dtypes),
         allow_unsafe_jscode=allow_unsafe_jscode,
         columns_state=columns_state,
         custom_css=custom_css,
@@ -491,7 +526,8 @@ def AgGrid(
         theme=themeObj,
         debug=default_column_parameters.pop("debug", False),
         update_on=update_on,
-        use_json_serialization=use_json_serialization
+        use_json_serialization=use_json_serialization,
+        server_sync_strategy=server_sync_strategy,
     )
 
     try:
@@ -511,7 +547,7 @@ def AgGrid(
                 f"PyArrow conversion failed, automatically retrying with JSON serialization: {error_msg}"
             )
             # Retry with JSON serialization enabled
-            _component_func_args['use_json_serialization'] = True
+            _component_func_args["use_json_serialization"] = True
             return AgGrid(**_component_func_args)
         elif not use_json_serialization and data is not None and is_pyarrow_error:
             # User explicitly disabled JSON serialization, raise the PyArrow error
